@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 #include <string.h>
 #include <stdlib.h>
 #include <glib-object.h>
@@ -1326,7 +1327,23 @@ regress_test_garray_container_return (void)
 {
   GPtrArray *array;
 
-  array = g_ptr_array_new_with_free_func (g_object_unref);
+  array = g_ptr_array_new_with_free_func (g_free);
+  g_ptr_array_add (array, g_strdup ("regress"));
+
+  return array;
+}
+
+/**
+ * regress_test_garray_full_return:
+ *
+ * Returns: (transfer full) (type GLib.PtrArray) (element-type utf8):
+ */
+GPtrArray *
+regress_test_garray_full_return (void)
+{
+  GPtrArray *array;
+
+  array = g_ptr_array_new ();
   g_ptr_array_add (array, g_strdup ("regress"));
 
   return array;
@@ -1656,6 +1673,35 @@ regress_test_boxed_get_type (void)
   return our_type;
 }
 
+RegressTestBoxedB *
+regress_test_boxed_b_new (gint8 some_int8, glong some_long)
+{
+  RegressTestBoxedB *boxed;
+
+  boxed = g_slice_new (RegressTestBoxedB);
+  boxed->some_int8 = some_int8;
+  boxed->some_long = some_long;
+
+  return boxed;
+}
+
+RegressTestBoxedB *
+regress_test_boxed_b_copy (RegressTestBoxedB *boxed)
+{
+  return regress_test_boxed_b_new (boxed->some_int8, boxed->some_long);
+}
+
+static void
+regress_test_boxed_b_free (RegressTestBoxedB *boxed)
+{
+  g_slice_free (RegressTestBoxedB, boxed);
+}
+
+G_DEFINE_BOXED_TYPE(RegressTestBoxedB,
+                    regress_test_boxed_b,
+                    regress_test_boxed_b_copy,
+                    regress_test_boxed_b_free);
+
 G_DEFINE_TYPE(RegressTestObj, regress_test_obj, G_TYPE_OBJECT);
 
 enum
@@ -2842,7 +2888,22 @@ regress_test_multi_callback (RegressTestCallback callback)
     return sum;
 }
 
+/**
+ * regress_test_array_callback:
+ * @callback: (scope call):
+ *
+ **/
+int regress_test_array_callback (RegressTestCallbackArray callback)
+{
+  static const char *strings[] = { "one", "two", "three" };
+  static int ints[] = { -1, 0, 1, 2 };
+  int sum = 0;
 
+  sum += callback(ints, 4, strings, 3);
+  sum += callback(ints, 4, strings, 3);
+
+  return sum;
+}
 
 /**
  * regress_test_simple_callback:
@@ -3026,6 +3087,58 @@ regress_test_obj_new_callback (RegressTestCallbackUserData callback, gpointer us
   return g_object_new (REGRESS_TEST_TYPE_OBJ, NULL);
 }
 
+/**
+ * regress_test_hash_table_callback:
+ * @data: (element-type utf8 gint): GHashTable that gets passed to callback
+ * @callback: (scope call):
+ **/
+void
+regress_test_hash_table_callback (GHashTable *data, RegressTestCallbackHashtable callback)
+{
+  callback (data);
+}
+
+/**
+ * regress_test_gerror_callback:
+ * @callback: (scope call):
+ **/
+void
+regress_test_gerror_callback (RegressTestCallbackGError callback)
+{
+  GError *error;
+
+  error = g_error_new_literal (G_IO_ERROR,
+                               G_IO_ERROR_NOT_SUPPORTED,
+                               "regression test error");
+  callback (error);
+  g_error_free (error);
+}
+
+/**
+ * regress_test_null_gerror_callback:
+ * @callback: (scope call):
+ **/
+void
+regress_test_null_gerror_callback (RegressTestCallbackGError callback)
+{
+  callback (NULL);
+}
+
+/**
+ * regress_test_owned_gerror_callback:
+ * @callback: (scope call):
+ **/
+void
+regress_test_owned_gerror_callback (RegressTestCallbackOwnedGError callback)
+{
+  GError *error;
+
+  error = g_error_new_literal (G_IO_ERROR,
+                               G_IO_ERROR_PERMISSION_DENIED,
+                               "regression test owned error");
+  callback (error);
+}
+
 /* interface */
 
 static void
